<?php

namespace Drupal\webform_pardot\Plugin\WebformHandler;

use Drupal\webform_pardot\PardotHandlerInterface;

use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionConditionsValidatorInterface;
use Drupal\webform\webformSubmissionInterface;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Queue\QueueFactory;

use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form submission handler.
 *
 * @WebformHandler(
 *   id = "pardot_submission",
 *   label = @Translation("Submit data to Pardot"),
 *   category = @Translation("Form Handler"),
 *   description = @Translation("Submit data to Pardot via Pardot Form Handler"),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_SINGLE,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class PardotSubmission extends WebformHandlerBase {

  /**
   * Pardot integration handler.
   *
   * @var \Drupal\webform_pardot\PardotHandlerInterface
   */
  protected $pardotHandler;

  /**
   * Queue Factory.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * Constructs a WebformHandlerBase object.
   *
   * IMPORTANT:
   * Webform handlers are initialized and serialized when they are attached to a
   * webform. Make sure not include any services as a dependency injection
   * that directly connect to the database. This will prevent
   * "LogicException: The database connection is not serializable." exceptions
   * from being thrown when a form is serialized via an Ajax callback and/or
   * form build.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\webform\WebformSubmissionConditionsValidatorInterface $conditions_validator
   *   The webform submission conditions (#states) validator.
   * @param \Drupal\webform_pardot\PardotHandlerInterface $pardot_handler
   *   The pardot handler.
   * @param \Drupal\Core\Queue\QueueFactory $queue
   *   The queue factory.
   *
   * @see \Drupal\webform\Entity\Webform::getHandlers
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    LoggerChannelFactoryInterface $logger_factory,
    ConfigFactoryInterface $config_factory,
    EntityTypeManagerInterface $entity_type_manager,
    WebformSubmissionConditionsValidatorInterface $conditions_validator,
    PardotHandlerInterface $pardot_handler,
    QueueFactory $queue
  ) {
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $logger_factory,
      $config_factory,
      $entity_type_manager,
      $conditions_validator
    );

    $this->entityTypeManager = $entity_type_manager;
    $this->pardotHandler = $pardot_handler;
    $this->queueFactory = $queue;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('webform_submission.conditions_validator'),
      $container->get('webform_pardot.pardot_handler'),
      $container->get('queue')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(
    array $form,
    FormStateInterface $form_state
  ) {
    $form['pardot_url'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('Pardot post url'),
      '#return_value' => TRUE,
      '#default_value' => $this->configuration['pardot_url'],
    ];

    $form['pardot_fields_mapping'] = [
      '#type' => 'textarea',
      '#required' => FALSE,
      '#title' => $this->t('Mapping of webform fields to Pardot fields'),
      '#description' => $this->t(
        'If Pardot uses different keys, you can map them here. ' .
        'Any key not included will be left as it is found on the webform. ' .
        'A list of "webform_key|pardot_key" is expected for each line. ' .
        'Values can be extracted from complex webform elements by writing the' .
        ' path to a value, separated by ".". For example, "person.name|name".'
      ),
      '#return_value' => TRUE,
      '#default_value' => $this->configuration['pardot_fields_mapping'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::validateConfigurationForm($form, $form_state);

    $values = $form_state->getValues();
    $values['pardot_url'] = trim($values['pardot_url']);
    $form_state->setValues($values);
    if (!filter_var($values['pardot_url'], FILTER_VALIDATE_URL)) {
      $form_state->setErrorByName('pardot_url', $this->t('Pardot post url must be a valid URL.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(
    array &$form,
    FormStateInterface $form_state
  ) {
    parent::submitConfigurationForm($form, $form_state);
    $this->applyFormStateToConfiguration($form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'pardot_url' => NULL,
      'pardot_fields_mapping' => NULL,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    $configuration = $this->getConfiguration();
    $settings = $configuration['settings'];

    return [
      '#settings' => $settings,
    ] + parent::getSummary();
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(
    WebformSubmissionInterface $webform_submission,
    $update = TRUE
  ) {
    // Create pardot submission entity with status queued.
    $pardot_submission = $this->createPardotSubmission($webform_submission);

    // Add the pardot submission in a queue to post data pardot.
    $values['pardot_submission_id'] = $pardot_submission->id();
    $queue = $this->queueFactory->get('pardot_submission_queue');
    $queue->createItem($values);
  }

  /**
   * Creates a pardot submission entity.
   *
   * A pardot submission entity is created in status queued.
   *
   * @param Drupal\webform\webformSubmissionInterface $webform_submission
   *   The webform submission.
   *
   * @return Drupal\webform_pardot\Entity\PardotSubmission
   *   The pardot submission entity.
   */
  private function createPardotSubmission(
    webformSubmissionInterface $webform_submission
  ) {
    $pardot_submission = $this->entityTypeManager
      ->getStorage('pardot_submission')
      ->create([
        'webform_submission' => $webform_submission,
        'status' => $this->pardotHandler::PARDOT_QUEUED,
      ]);
    $pardot_submission->save();

    return $pardot_submission;
  }

}
